# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 5.15.2
# WARNING! All changes made in this file will be lost!

from Qt import QtCompat, QtCore

qt_resource_data = b"\
\x00\x00\x0bf\
\x89\
PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\
\x00\x00 \x00\x00\x00 \x08\x06\x00\x00\x00szz\xf4\
\x00\x00\x00\x09pHYs\x00\x00\x0b\x13\x00\x00\x0b\x13\
\x01\x00\x9a\x9c\x18\x00\x00\x06\x98iTXtXML\
:com.adobe.xmp\x00\x00\
\x00\x00\x00<?xpacket beg\
in=\x22\xef\xbb\xbf\x22 id=\x22W5M\
0MpCehiHzreSzNTc\
zkc9d\x22?> <x:xmpm\
eta xmlns:x=\x22ado\
be:ns:meta/\x22 x:x\
mptk=\x22Adobe XMP \
Core 5.6-c142 79\
.160924, 2017/07\
/13-01:06:39    \
    \x22> <rdf:RDF \
xmlns:rdf=\x22http:\
//www.w3.org/199\
9/02/22-rdf-synt\
ax-ns#\x22> <rdf:De\
scription rdf:ab\
out=\x22\x22 xmlns:xmp\
=\x22http://ns.adob\
e.com/xap/1.0/\x22 \
xmlns:dc=\x22http:/\
/purl.org/dc/ele\
ments/1.1/\x22 xmln\
s:photoshop=\x22htt\
p://ns.adobe.com\
/photoshop/1.0/\x22\
 xmlns:xmpMM=\x22ht\
tp://ns.adobe.co\
m/xap/1.0/mm/\x22 x\
mlns:stEvt=\x22http\
://ns.adobe.com/\
xap/1.0/sType/Re\
sourceEvent#\x22 xm\
p:CreatorTool=\x22A\
dobe Photoshop C\
C (Windows)\x22 xmp\
:CreateDate=\x22201\
8-03-08T11:12:25\
Z\x22 xmp:ModifyDat\
e=\x222018-03-08T11\
:41:22Z\x22 xmp:Met\
adataDate=\x222018-\
03-08T11:41:22Z\x22\
 dc:format=\x22imag\
e/png\x22 photoshop\
:ColorMode=\x223\x22 p\
hotoshop:ICCProf\
ile=\x22sRGB IEC619\
66-2.1\x22 xmpMM:In\
stanceID=\x22xmp.ii\
d:48dadb43-ebb4-\
104e-9bdd-32a1de\
467c44\x22 xmpMM:Do\
cumentID=\x22adobe:\
docid:photoshop:\
8f60f140-c3d4-a3\
4c-87ce-b8474961\
44b4\x22 xmpMM:Orig\
inalDocumentID=\x22\
xmp.did:2a58b225\
-2e75-e94d-bdee-\
4030865c5daf\x22> <\
xmpMM:History> <\
rdf:Seq> <rdf:li\
 stEvt:action=\x22c\
reated\x22 stEvt:in\
stanceID=\x22xmp.ii\
d:2a58b225-2e75-\
e94d-bdee-403086\
5c5daf\x22 stEvt:wh\
en=\x222018-03-08T1\
1:12:25Z\x22 stEvt:\
softwareAgent=\x22A\
dobe Photoshop C\
C (Windows)\x22/> <\
rdf:li stEvt:act\
ion=\x22saved\x22 stEv\
t:instanceID=\x22xm\
p.iid:44467f4f-1\
dee-8541-bd3d-00\
2a4e7d424d\x22 stEv\
t:when=\x222018-03-\
08T11:14:34Z\x22 st\
Evt:softwareAgen\
t=\x22Adobe Photosh\
op CC (Windows)\x22\
 stEvt:changed=\x22\
/\x22/> <rdf:li stE\
vt:action=\x22saved\
\x22 stEvt:instance\
ID=\x22xmp.iid:48da\
db43-ebb4-104e-9\
bdd-32a1de467c44\
\x22 stEvt:when=\x2220\
18-03-08T11:41:2\
2Z\x22 stEvt:softwa\
reAgent=\x22Adobe P\
hotoshop CC (Win\
dows)\x22 stEvt:cha\
nged=\x22/\x22/> </rdf\
:Seq> </xmpMM:Hi\
story> </rdf:Des\
cription> </rdf:\
RDF> </x:xmpmeta\
> <?xpacket end=\
\x22r\x22?>\xfd\xf0\xcc\x85\x00\x00\x04tIDA\
TX\xc3\xa5\x97{\x88UU\x14\xc6\x7f\xf74\xe6\xe4\
(N\xcc\xa8e\x9ai\xaf\x89\x99\x1c\xcd\xd0\x18{\x11\
\xa5\x95Vb#\xf6P\xb2\xb2$($\xb50\x09\x0a\
\x0b\x22zL\xe5dDYVH\xa9=\xa6\xfc\xc3\x86\
\x094{`/g\xba\x96\xa5\x93\x1aj:j4e\
\x8d\x98\xa3\xdc\xfe\xf96|l\xce\xbd]q\xc1\xe1\xec\
\xbd\xce>{\xad\xbd\xbe\xf5\xda\x99Q-Y\x8a\xa4>\
\xc0\xb5\xc0X`\x040\x08\xe8\xabo\x07\x80]@\x16\
\xf8\x02X\x0dt\x16\xb3i\xa6\x08\x05\x86\x01\xf7\x037\
\x03\x15E*{\x00x\x07x\x0e\xf8\xe9x\x14X\x08\
<\x0cd\x22\xfe\x16\xe0\x17\xa0\x03\xc8\x01\x03\x803\x81\
\xf3R\xf6x\x0ax0\x9f\x80\x92<\xfcS\x81&`\
\xb4\xf1\xbe\x05\x96\x02\xcd\x12\x9eFC\x81\xab\x80\xdb\x80\
:\xf1\x1e\x00\xae\x06n\x00\xb6\x17c\x81a\xc0z\xa0\
\x9f\xe6\xbb\x80y\xc0r\x8e\x8d\xae\x07\x9e\x01\xce\xd2\xfc\
\x1f\xf9O\xb6\x90\x02\x15@;p\xb2\xe6M\xc0\xad\xc0\
A[s90\x0e\x18\x0eT\x8a\xb7\x1f\xf8\x0ex\x01\
\xf8#\xb2\xf0R\xed\x01p\x088[\x87J\x85`\xad\
\x09_\x02\xcc\xb4oS\x80\x87\x80\x91yN<\x11\xa8\
\x01\xea\x8dw\x04\x98&_\x99\x0b\x94\x02\x9f\x09*\x00\
\x92\xc8\xe1j4^\x1d\x09_\x06\xac\x88\x84\xef\x00>\
\xd5\xb3O\xc2\xd6\xe6Qn\x1e\xf0\xb6\xc6g\x00\x8bb\
\x08\x06\x02\xbf\x89\xd7)(r\x9a\xaf\x03.\xb1\xcd\x1a\
\x80\xb7\x806[S\x0a\x94\xeb\xa4\x08\xf7\xe5\x82\xe3F\
\x85%\xc0N\xe5\x0f\x145\xdb\x82\x05\x1e1\x013l\
\xe3\x15&|\x13P\x0d\xcc\x01ZmM\xc0\xb6\xc3\xe6\
\xe3\x80\x0b\x80+\xcd\x091_\x00x\x22X\xa0\x17\xf0\
'\xd0\x03\xd8\x0cT\x19\xa6\xab4n\x97\xf0n\xcd\xcb\
\x80\xc9\x12\x92\x915>\x00\xfe\xd2\xf7r\x99\xb9S\x0a\
\x1f1\xc1_YxW\x96(\xbd\xf60\xf3\x06Zl\
\xe3+L\xf8\x14\xa0\x11\xe8\x1f\xe1\xfc\xb4\x1c\xed\x0d\x1d\
hz\x1e\x7fxVY\x12`R\x223\x05\xfaH\xef\
\xcb\x80\xc1\x1a/\xb2\xb0\x99(X\x82\xf0\xa3\xa6X\x85\
B\xee\x96H\xe0@A\x829\xf8\xd1p\xb0\xc4<{\
\x0b\xb0G\xe3\xeb\xec\x87\xe7\xf5\xee\x05\xbco\xfc\xf9\xc0\
\xe9z\xe6D\x11SnP\xb5*{\xdemu\xa2U\
\xe3\xdaD\x1a\x02\xfc`\x9b\xd4Z\xa8m\xd5\xf8&\x83\
j\x01\xf0$\xb0[\xce\xd7\x00\xdck\xff\xdfna\x1e\
2\xead\xfb\xfe\xb3\xde\xfd\x13\x95Y\xa2\x0c\x162\xdc\
\xaf\xc6\x0b\xb9\xfd_\xe1\x1d\xd3\x8b\xc2\x1e\xe0B\xbd\xff\
V\x0d\xc1\xac\x122'@\xcf\xe4\x18r{8\xfd\x89\
\xaa~\x05\xabl\x0a/\x976N\xa4%\x96\x82\x01~\
\xb7\xea\x16\xe8\x13\xdb\xfc\xa4\x14\x013\xed\x94\x1blm\
\xc8\x03{lm\x80\xe5p\x89p\x1c\xa48\x0f\xd4\xa6\
\xe8\x18\xac\xe2\xd1\xae\xec\xd7\x17\xd8\xab\xb9\xc3U\x0f\xbc\
d\xbc\xd7\xf4\x1ei\x07\xf3\xaaw\xae\xde\xfb\x12\xe0{\
M\xaa\xcc\xb4\x1f\xda\xe2\xb96n\x04Vj<A>\
\xb23\x12~\x87\xf9\x93GG\xd8\xb3\xb7E^61\
\xd3\x868\x07\xf8\xdc\x9a\x87Y\xea\x11b\xaa\x07\x86\xa8\
\x0e\x00t\x01\xf7\x00\xafk~\xbe\xa5\xde\x8d\x16z\xe3\
\xcd\x9f\xd6dF\xb5d\xcb\x94BO\x00~\x94v\xdd\
Z\xf8\xb1\x15\x91j\xf3\x17\x80ST\x9e\xcb\x84\xf9{\
\x82'\xc0\xb2\xc9\xb0\xbeX\xcd*z\xd7y\x18v\x01\
o\x8aQ\x0d\x9c\xa3q\xb3pG\xbe\xd0\x16\x95\xe3\x0e\
`\xb6\x9co\xb1\x09\xaf\xd3\xda~\x96\xd2\x83\xf0\x8bL\
x\x13\xb0?\x94\xe3J\x9d\xb6\x03\x98*\xa5\x02\xb5D\
\xe9\xfae)\xf6\x0dp8\xc4\xb36\x9f\xa1'\xd0*\
\xb5f\x81\xb6YdU\x01\x9b3E\xde\x0b\x96\xc8\xb9\
\x9c\xf6Z\xa2\x1a\x9aR\x9c\x1a\x81\xfb\xa2\xc8\x08\x19\xf2\
U\xe0\xae\xb8#\xf2\x84\xf3h\xe4\xfdw\xaa\xab]o\
\xbc\x01\xc0\x18=.|\x9d\xba`\x17\xfe\xb8\x09\xdfm\
u!\xb5+\x9ef\xd8\x8f\x05\xbe\x8c\xbe\x8f\x96\x83\xd6\
\xa8}\x0f\x9bf\x05\xe3\x86h\xfd+\xd6\xde\xe5\x94W\
\xb6\x16\xba\x17d\xd5\xe3\x1dT1\x8a\xe9k=\xffG\
\xe3U3j\xacA\xbd\xd4\x85\x17R`\x88\x8aNN\
\xe9t\x81B\xe9]9\xe5\x8e<BO\xd3\xc5d\xba\
\x9a\x18\xbfIMJ\xbb\xa6\x15\xe3\x84}\xac\xa9\x0cf\
\xcc*\x1d\xc7W\xb3\x11\xca'q\x95\x9cmMHQ\
W3\xa7.\xe01\xa5\xd52Y\xa4\xd6z\x864\xea\
V\xe7\xd4\xa0\x0b\xcbq\xdd\x8e\xbd\x82]#\x1c\x87\xab\
\x13*\x95B\x87\xd4\xb6mT\x144[\x9b_\x90\xfe\
\x03=x\x1ffGD\xffn\x00\x00\x00\x00IEN\
D\xaeB`\x82\
"

qt_resource_name = b"\
\x00\x05\
\x00o\xa6S\
\x00i\
\x00c\x00o\x00n\x00s\
\x00\x0a\
\x0ajx\xa7\
\x00T\
\x00S\x00I\x00c\x00o\x00n\x00.\x00p\x00n\x00g\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x03\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x10\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01pR\xb6\xfcv\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
